@@ -1,27 +1,44 @@
 """
 Quantilization functions and related stuff
 """
-from functools import partial
-
 import numpy as np
 
+from pandas._libs import Timedelta, Timestamp
 from pandas._libs.lib import infer_dtype
 
 from pandas.core.dtypes.common import (
-    _NS_DTYPE, ensure_int64, is_categorical_dtype, is_datetime64_dtype,
-    is_datetime64tz_dtype, is_datetime_or_timedelta_dtype, is_integer,
-    is_scalar, is_timedelta64_dtype)
+    _NS_DTYPE,
+    ensure_int64,
+    is_bool_dtype,
+    is_categorical_dtype,
+    is_datetime64_dtype,
+    is_datetime64tz_dtype,
+    is_datetime_or_timedelta_dtype,
+    is_extension_array_dtype,
+    is_integer,
+    is_integer_dtype,
+    is_list_like,
+    is_scalar,
+    is_timedelta64_dtype,
+)
+from pandas.core.dtypes.generic import ABCSeries
 from pandas.core.dtypes.missing import isna
 
-from pandas import (
-    Categorical, Index, Interval, IntervalIndex, Series, Timedelta, Timestamp,
-    to_datetime, to_timedelta)
+from pandas import Categorical, Index, IntervalIndex, to_datetime, to_timedelta
 import pandas.core.algorithms as algos
 import pandas.core.nanops as nanops
 
 
-def cut(x, bins, right=True, labels=None, retbins=False, precision=3,
-        include_lowest=False, duplicates='raise'):
+def cut(
+    x,
+    bins,
+    right: bool = True,
+    labels=None,
+    retbins: bool = False,
+    precision: int = 3,
+    include_lowest: bool = False,
+    duplicates: str = "raise",
+):
     """
     Bin values into discrete intervals.
 
@@ -35,7 +52,7 @@ def cut(x, bins, right=True, labels=None, retbins=False, precision=3,
     ----------
     x : array-like
         The input array to be binned. Must be 1-dimensional.
-    bins : int, sequence of scalars, or pandas.IntervalIndex
+    bins : int, sequence of scalars, or IntervalIndex
         The criteria to bin by.
 
         * int : Defines the number of equal-width bins in the range of `x`. The
@@ -51,11 +68,12 @@ def cut(x, bins, right=True, labels=None, retbins=False, precision=3,
         ``right == True`` (the default), then the `bins` ``[1, 2, 3, 4]``
         indicate (1,2], (2,3], (3,4]. This argument is ignored when
         `bins` is an IntervalIndex.
-    labels : array or bool, optional
+    labels : array or False, default None
         Specifies the labels for the returned bins. Must be the same length as
         the resulting bins. If False, returns only integer indicators of the
         bins. This affects the type of the output container (see below).
-        This argument is ignored when `bins` is an IntervalIndex.
+        This argument is ignored when `bins` is an IntervalIndex. If True,
+        raises an error.
     retbins : bool, default False
         Whether to return the bins or not. Useful when bins is provided
         as a scalar.
@@ -70,16 +88,16 @@ def cut(x, bins, right=True, labels=None, retbins=False, precision=3,
 
     Returns
     -------
-    out : pandas.Categorical, Series, or ndarray
+    out : Categorical, Series, or ndarray
         An array-like object representing the respective bin for each value
         of `x`. The type depends on the value of `labels`.
 
         * True (default) : returns a Series for Series `x` or a
-          pandas.Categorical for all other inputs. The values stored within
+          Categorical for all other inputs. The values stored within
           are Interval dtype.
 
         * sequence of scalars : returns a Series for Series `x` or a
-          pandas.Categorical for all other inputs. The values stored within
+          Categorical for all other inputs. The values stored within
           are whatever the type in the sequence is.
 
         * False : returns an ndarray of integers.
@@ -94,16 +112,15 @@ def cut(x, bins, right=True, labels=None, retbins=False, precision=3,
     --------
     qcut : Discretize variable into equal-sized buckets based on rank
         or based on sample quantiles.
-    pandas.Categorical : Array type for storing data that come from a
+    Categorical : Array type for storing data that come from a
         fixed set of values.
     Series : One-dimensional array with axis labels (including time series).
-    pandas.IntervalIndex : Immutable Index implementing an ordered,
-        sliceable set.
+    IntervalIndex : Immutable Index implementing an ordered, sliceable set.
 
     Notes
     -----
     Any NA values will be NA in the result. Out of bounds values will be NA in
-    the resulting Series or pandas.Categorical object.
+    the resulting Series or Categorical object.
 
     Examples
     --------
@@ -164,7 +181,7 @@ def cut(x, bins, right=True, labels=None, retbins=False, precision=3,
     Use `drop` optional when bins is not unique
 
     >>> pd.cut(s, [0, 2, 4, 6, 10, 10], labels=False, retbins=True,
-    ...    right=False, duplicates='drop')
+    ...        right=False, duplicates='drop')
     ... # doctest: +ELLIPSIS
     (a    0.0
      b    1.0
@@ -186,9 +203,16 @@ def cut(x, bins, right=True, labels=None, retbins=False, precision=3,
     # NOTE: this binning code is changed a bit from histogram for var(x) == 0
 
     # for handling the cut for datetime and timedelta objects
-    x_is_series, series_index, name, x = _preprocess_for_cut(x)
+    original = x
+    x = _preprocess_for_cut(x)
     x, dtype = _coerce_to_type(x)
 
+    # To support cut(IntegerArray), we convert to object dtype with NaN
+    # Will properly support in the future.
+    # https://github.com/pandas-dev/pandas/pull/31290
+    if is_extension_array_dtype(x.dtype) and is_integer_dtype(x.dtype):
+        x = x.to_numpy(dtype=object, na_value=np.nan)
+
     if not np.iterable(bins):
         if is_scalar(bins) and bins < 1:
             raise ValueError("`bins` should be a positive integer.")
@@ -200,18 +224,19 @@ def cut(x, bins, right=True, labels=None, retbins=False, precision=3,
             sz = x.size
 
         if sz == 0:
-            raise ValueError('Cannot cut empty array')
+            raise ValueError("Cannot cut empty array")
 
         rng = (nanops.nanmin(x), nanops.nanmax(x))
         mn, mx = [mi + 0.0 for mi in rng]
 
         if np.isinf(mn) or np.isinf(mx):
             # GH 24314
-            raise ValueError('cannot specify integer `bins` when input data '
-                             'contains infinity')
+            raise ValueError(
+                "cannot specify integer `bins` when input data contains infinity"
+            )
         elif mn == mx:  # adjust end points before binning
-            mn -= .001 * abs(mn) if mn != 0 else .001
-            mx += .001 * abs(mx) if mx != 0 else .001
+            mn -= 0.001 * abs(mn) if mn != 0 else 0.001
+            mx += 0.001 * abs(mx) if mx != 0 else 0.001
             bins = np.linspace(mn, mx, bins + 1, endpoint=True)
         else:  # adjust end points after binning
             bins = np.linspace(mn, mx, bins + 1, endpoint=True)
@@ -223,7 +248,7 @@ def cut(x, bins, right=True, labels=None, retbins=False, precision=3,
 
     elif isinstance(bins, IntervalIndex):
         if bins.is_overlapping:
-            raise ValueError('Overlapping IntervalIndex is not accepted.')
+            raise ValueError("Overlapping IntervalIndex is not accepted.")
 
     else:
         if is_datetime64tz_dtype(bins):
@@ -231,46 +256,58 @@ def cut(x, bins, right=True, labels=None, retbins=False, precision=3,
         else:
             bins = np.asarray(bins)
         bins = _convert_bin_to_numeric_type(bins, dtype)
-        if (np.diff(bins) < 0).any():
-            raise ValueError('bins must increase monotonically.')
 
-    fac, bins = _bins_to_cuts(x, bins, right=right, labels=labels,
-                              precision=precision,
-                              include_lowest=include_lowest,
-                              dtype=dtype,
-                              duplicates=duplicates)
+        # GH 26045: cast to float64 to avoid an overflow
+        if (np.diff(bins.astype("float64")) < 0).any():
+            raise ValueError("bins must increase monotonically.")
 
-    return _postprocess_for_cut(fac, bins, retbins, x_is_series,
-                                series_index, name, dtype)
+    fac, bins = _bins_to_cuts(
+        x,
+        bins,
+        right=right,
+        labels=labels,
+        precision=precision,
+        include_lowest=include_lowest,
+        dtype=dtype,
+        duplicates=duplicates,
+    )
+
+    return _postprocess_for_cut(fac, bins, retbins, dtype, original)
 
 
-def qcut(x, q, labels=None, retbins=False, precision=3, duplicates='raise'):
+def qcut(
+    x,
+    q,
+    labels=None,
+    retbins: bool = False,
+    precision: int = 3,
+    duplicates: str = "raise",
+):
     """
-    Quantile-based discretization function. Discretize variable into
-    equal-sized buckets based on rank or based on sample quantiles. For example
-    1000 values for 10 quantiles would produce a Categorical object indicating
-    quantile membership for each data point.
+    Quantile-based discretization function.
+
+    Discretize variable into equal-sized buckets based on rank or based
+    on sample quantiles. For example 1000 values for 10 quantiles would
+    produce a Categorical object indicating quantile membership for each data point.
 
     Parameters
     ----------
     x : 1d ndarray or Series
-    q : integer or array of quantiles
+    q : int or list-like of int
         Number of quantiles. 10 for deciles, 4 for quartiles, etc. Alternately
-        array of quantiles, e.g. [0, .25, .5, .75, 1.] for quartiles
-    labels : array or boolean, default None
+        array of quantiles, e.g. [0, .25, .5, .75, 1.] for quartiles.
+    labels : array or False, default None
         Used as labels for the resulting bins. Must be of the same length as
         the resulting bins. If False, return only integer indicators of the
-        bins.
+        bins. If True, raises an error.
     retbins : bool, optional
         Whether to return the (bins, labels) or not. Can be useful if bins
         is given as a scalar.
     precision : int, optional
-        The precision at which to store and display the bins labels
+        The precision at which to store and display the bins labels.
     duplicates : {default 'raise', 'drop'}, optional
         If bin edges are not unique, raise ValueError or drop non-uniques.
 
-        .. versionadded:: 0.20.0
-
     Returns
     -------
     out : Categorical or Series or array of integers if labels is False
@@ -299,8 +336,8 @@ def qcut(x, q, labels=None, retbins=False, precision=3, duplicates='raise'):
     >>> pd.qcut(range(5), 4, labels=False)
     array([0, 0, 1, 2, 3])
     """
-    x_is_series, series_index, name, x = _preprocess_for_cut(x)
-
+    original = x
+    x = _preprocess_for_cut(x)
     x, dtype = _coerce_to_type(x)
 
     if is_integer(q):
@@ -308,39 +345,53 @@ def qcut(x, q, labels=None, retbins=False, precision=3, duplicates='raise'):
     else:
         quantiles = q
     bins = algos.quantile(x, quantiles)
-    fac, bins = _bins_to_cuts(x, bins, labels=labels,
-                              precision=precision, include_lowest=True,
-                              dtype=dtype, duplicates=duplicates)
+    fac, bins = _bins_to_cuts(
+        x,
+        bins,
+        labels=labels,
+        precision=precision,
+        include_lowest=True,
+        dtype=dtype,
+        duplicates=duplicates,
+    )
 
-    return _postprocess_for_cut(fac, bins, retbins, x_is_series,
-                                series_index, name, dtype)
+    return _postprocess_for_cut(fac, bins, retbins, dtype, original)
 
 
-def _bins_to_cuts(x, bins, right=True, labels=None,
-                  precision=3, include_lowest=False,
-                  dtype=None, duplicates='raise'):
+def _bins_to_cuts(
+    x,
+    bins,
+    right: bool = True,
+    labels=None,
+    precision: int = 3,
+    include_lowest: bool = False,
+    dtype=None,
+    duplicates: str = "raise",
+):
 
-    if duplicates not in ['raise', 'drop']:
-        raise ValueError("invalid value for 'duplicates' parameter, "
-                         "valid options are: raise, drop")
+    if duplicates not in ["raise", "drop"]:
+        raise ValueError(
+            "invalid value for 'duplicates' parameter, "
+            "valid options are: raise, drop"
+        )
 
     if isinstance(bins, IntervalIndex):
         # we have a fast-path here
         ids = bins.get_indexer(x)
-        result = algos.take_nd(bins, ids)
-        result = Categorical(result, categories=bins, ordered=True)
+        result = Categorical.from_codes(ids, categories=bins, ordered=True)
         return result, bins
 
     unique_bins = algos.unique(bins)
     if len(unique_bins) < len(bins) and len(bins) != 2:
-        if duplicates == 'raise':
-            raise ValueError("Bin edges must be unique: {bins!r}.\nYou "
-                             "can drop duplicate edges by setting "
-                             "the 'duplicates' kwarg".format(bins=bins))
+        if duplicates == "raise":
+            raise ValueError(
+                f"Bin edges must be unique: {repr(bins)}.\n"
+                f"You can drop duplicate edges by setting the 'duplicates' kwarg"
+            )
         else:
             bins = unique_bins
 
-    side = 'left' if right else 'right'
+    side = "left" if right else "right"
     ids = ensure_int64(bins.searchsorted(x, side=side))
 
     if include_lowest:
@@ -350,14 +401,23 @@ def _bins_to_cuts(x, bins, right=True, labels=None,
     has_nas = na_mask.any()
 
     if labels is not False:
-        if labels is None:
-            labels = _format_labels(bins, precision, right=right,
-                                    include_lowest=include_lowest,
-                                    dtype=dtype)
+        if not (labels is None or is_list_like(labels)):
+            raise ValueError(
+                "Bin labels must either be False, None or passed in as a "
+                "list-like argument"
+            )
+
+        elif labels is None:
+            labels = _format_labels(
+                bins, precision, right=right, include_lowest=include_lowest, dtype=dtype
+            )
+
         else:
             if len(labels) != len(bins) - 1:
-                raise ValueError('Bin labels must be one fewer than '
-                                 'the number of bin edges')
+                raise ValueError(
+                    "Bin labels must be one fewer than the number of bin edges"
+                )
+
         if not is_categorical_dtype(labels):
             labels = Categorical(labels, categories=labels, ordered=True)
 
@@ -373,18 +433,10 @@ def _bins_to_cuts(x, bins, right=True, labels=None,
     return result, bins
 
 
-def _trim_zeros(x):
-    while len(x) > 1 and x[-1] == '0':
-        x = x[:-1]
-    if len(x) > 1 and x[-1] == '.':
-        x = x[:-1]
-    return x
-
-
 def _coerce_to_type(x):
     """
-    if the passed data is of datetime/timedelta type,
-    this method converts it to numeric so that cut method can
+    if the passed data is of datetime/timedelta or bool type,
+    this method converts it to numeric so that cut or qcut method can
     handle it
     """
     dtype = None
@@ -393,10 +445,13 @@ def _coerce_to_type(x):
         dtype = x.dtype
     elif is_datetime64_dtype(x):
         x = to_datetime(x)
-        dtype = np.dtype('datetime64[ns]')
+        dtype = np.dtype("datetime64[ns]")
     elif is_timedelta64_dtype(x):
         x = to_timedelta(x)
-        dtype = np.dtype('timedelta64[ns]')
+        dtype = np.dtype("timedelta64[ns]")
+    elif is_bool_dtype(x):
+        # GH 20303
+        x = x.astype(np.int64)
 
     if dtype is not None:
         # GH 19768: force NaT to NaN during integer conversion
@@ -421,12 +476,12 @@ def _convert_bin_to_numeric_type(bins, dtype):
     """
     bins_dtype = infer_dtype(bins, skipna=False)
     if is_timedelta64_dtype(dtype):
-        if bins_dtype in ['timedelta', 'timedelta64']:
+        if bins_dtype in ["timedelta", "timedelta64"]:
             bins = to_timedelta(bins).view(np.int64)
         else:
             raise ValueError("bins must be of timedelta64 dtype")
     elif is_datetime64_dtype(dtype) or is_datetime64tz_dtype(dtype):
-        if bins_dtype in ['datetime', 'datetime64']:
+        if bins_dtype in ["datetime", "datetime64"]:
             bins = to_datetime(bins).view(np.int64)
         else:
             raise ValueError("bins must be of datetime64 dtype")
@@ -436,7 +491,7 @@ def _convert_bin_to_numeric_type(bins, dtype):
 
 def _convert_bin_to_datelike_type(bins, dtype):
     """
-    Convert bins to a DatetimeIndex or TimedeltaIndex if the orginal dtype is
+    Convert bins to a DatetimeIndex or TimedeltaIndex if the original dtype is
     datelike
 
     Parameters
@@ -450,45 +505,39 @@ def _convert_bin_to_datelike_type(bins, dtype):
            datelike
     """
     if is_datetime64tz_dtype(dtype):
-        bins = to_datetime(bins.astype(np.int64),
-                           utc=True).tz_convert(dtype.tz)
+        bins = to_datetime(bins.astype(np.int64), utc=True).tz_convert(dtype.tz)
     elif is_datetime_or_timedelta_dtype(dtype):
         bins = Index(bins.astype(np.int64), dtype=dtype)
     return bins
 
 
-def _format_labels(bins, precision, right=True,
-                   include_lowest=False, dtype=None):
+def _format_labels(
+    bins, precision: int, right: bool = True, include_lowest: bool = False, dtype=None
+):
     """ based on the dtype, return our labels """
 
-    closed = 'right' if right else 'left'
+    closed = "right" if right else "left"
 
     if is_datetime64tz_dtype(dtype):
-        formatter = partial(Timestamp, tz=dtype.tz)
-        adjust = lambda x: x - Timedelta('1ns')
+        formatter = lambda x: Timestamp(x, tz=dtype.tz)
+        adjust = lambda x: x - Timedelta("1ns")
     elif is_datetime64_dtype(dtype):
         formatter = Timestamp
-        adjust = lambda x: x - Timedelta('1ns')
+        adjust = lambda x: x - Timedelta("1ns")
     elif is_timedelta64_dtype(dtype):
         formatter = Timedelta
-        adjust = lambda x: x - Timedelta('1ns')
+        adjust = lambda x: x - Timedelta("1ns")
     else:
         precision = _infer_precision(precision, bins)
         formatter = lambda x: _round_frac(x, precision)
         adjust = lambda x: x - 10 ** (-precision)
 
     breaks = [formatter(b) for b in bins]
-    labels = IntervalIndex.from_breaks(breaks, closed=closed)
-
     if right and include_lowest:
-        # we will adjust the left hand side by precision to
-        # account that we are all right closed
-        v = adjust(labels[0].left)
+        # adjust lhs of first interval by precision to account for being right closed
+        breaks[0] = adjust(breaks[0])
 
-        i = IntervalIndex([Interval(v, labels[0].right, closed='right')])
-        labels = i.append(labels[1:])
-
-    return labels
+    return IntervalIndex.from_breaks(breaks, closed=closed)
 
 
 def _preprocess_for_cut(x):
@@ -497,34 +546,26 @@ def _preprocess_for_cut(x):
     input to array, strip the index information and store it
     separately
     """
-    x_is_series = isinstance(x, Series)
-    series_index = None
-    name = None
-
-    if x_is_series:
-        series_index = x.index
-        name = x.name
 
     # Check that the passed array is a Pandas or Numpy object
     # We don't want to strip away a Pandas data-type here (e.g. datetimetz)
-    ndim = getattr(x, 'ndim', None)
+    ndim = getattr(x, "ndim", None)
     if ndim is None:
         x = np.asarray(x)
     if x.ndim != 1:
         raise ValueError("Input array must be 1 dimensional")
 
-    return x_is_series, series_index, name, x
+    return x
 
 
-def _postprocess_for_cut(fac, bins, retbins, x_is_series,
-                         series_index, name, dtype):
+def _postprocess_for_cut(fac, bins, retbins: bool, dtype, original):
     """
     handles post processing for the cut method where
     we combine the index information if the originally passed
     datatype was a series
     """
-    if x_is_series:
-        fac = Series(fac, index=series_index, name=name)
+    if isinstance(original, ABCSeries):
+        fac = original._constructor(fac, index=original.index, name=original.name)
 
     if not retbins:
         return fac
@@ -534,7 +575,7 @@ def _postprocess_for_cut(fac, bins, retbins, x_is_series,
     return fac, bins
 
 
-def _round_frac(x, precision):
+def _round_frac(x, precision: int):
     """
     Round the fractional part of the given number
     """
@@ -549,7 +590,7 @@ def _round_frac(x, precision):
         return np.around(x, digits)
 
 
-def _infer_precision(base_precision, bins):
+def _infer_precision(base_precision: int, bins) -> int:
     """Infer an appropriate precision for _round_frac
     """
     for precision in range(base_precision, 20):
